SystemOrganization addCategory: #'FuelBenchmarks-Samples'!
SystemOrganization addCategory: #'FuelBenchmarks-Tests'!
SystemOrganization addCategory: #'FuelBenchmarks-Other'!
SystemOrganization addCategory: #'FuelBenchmarks-Core'!
SystemOrganization addCategory: #'FuelBenchmarks-Runners'!

!Collection class methodsFor: '*FuelBenchmarks' stamp: 'MartinDias 4/9/2012 06:23'!
denseSample
	
	| desiredSize |
	desiredSize := (self new capacity * 0.85) floor.
	^ self new 
		addAll: (1 to: desiredSize); 
		yourself! !

!Collection class methodsFor: '*FuelBenchmarks' stamp: 'MartinDias 4/9/2012 06:25'!
sparseSample
	
	| desiredSize |
	desiredSize := (self new capacity * 0.25) floor.
	^ self new 
		addAll: (1 to: desiredSize); 
		yourself! !

TestCase subclass: #FLSampleFactoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLSampleFactoryTest commentStamp: '<historical>' prior: 0!
A FLSampleFactoryTest is a test class for testing the behavior of FLSampleFactory!

!FLSampleFactoryTest methodsFor: 'tests' stamp: 'MartinDias 4/12/2012 17:00'!
testAsSample

	| sample |
	sample := FLNumberSampleFactory new
		selector: #smallIntegers;
		asSample.
	
	self assert: sample title = 'smallIntegers'.
	self deny: sample content isEmpty.
	self assert: sample content anyOne isInteger.! !

!FLSampleFactoryTest methodsFor: 'tests' stamp: 'MartinDias 4/12/2012 17:01'!
testProblemOfSmallSize
	"Problem size aproximately states the size of the built sample."

	| factory smallSample bigSample |
	factory := FLNumberSampleFactory new
		selector: #smallIntegers.
	
	smallSample := factory problemSize: 1; asSample.
	bigSample := factory problemSize: 100; asSample.

	self assert: smallSample content size < bigSample content size.
! !

FLSerializationTest subclass: #FLBenchmarkSampleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarkSampleTest methodsFor: 'tests' stamp: 'MartinDias 4/12/2012 01:40'!
testAllSamples
	"Tests that the result of serialization and materialization of every sample does not raise an error and that it has the expected size. Would be nice to check if the result is equal but many classes do not have = well defined for us."

	FLSampleFactory allSubclasses do: [ :aFactory |
		aFactory allSamples do: [ :aSample | 
			self testSample: aSample ] ]! !

!FLBenchmarkSampleTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:18'!
testSample: aSample
	"Tests that the result of serialization and materialization has the expected size. Would be nice to check if the result is equal but many classes do not have = well defined for us."

	| result |
	self 
		shouldnt: [ result := self resultOfSerializeAndMaterialize: aSample content ] 
		raise: Error.
	
	self assert: aSample content size = result size.! !

FLSerializationTest subclass: #FLBenchmarksTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarksTest classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 10/31/2011 12:03'!
packageNamesUnderTest
	^ #('FuelBenchmarks')! !

!FLBenchmarksTest methodsFor: 'running' stamp: 'MartinDias 6/2/2011 19:34'!
benchmarksWithTwoRunnersAndTwoSamples
	
	| benchmarks |
	benchmarks := FLBenchmarks new.
	benchmarks runners add: (FLBenchmarksDummyRunner titled: 'A').
	benchmarks runners add: (FLBenchmarksDummyRunner titled: 'B').
	benchmarks samples add: (FLBenchmarkSample title: 'sample1' content: #(1 2 3 4)).
	benchmarks samples add: (FLBenchmarkSample title: 'sample2' content: #(5 6 7 8)).
	^ benchmarks! !

!FLBenchmarksTest methodsFor: 'running' stamp: 'MartinDias 4/12/2012 02:14'!
fuelBenchmarksWithSmallSample
	^ FLBenchmarks new
		samples: { FLNumberSampleFactory new selector: #smallIntegers; problemSize: 1; asSample };
		runners: { FLFuelBenchmarkRunner new };
		yourself! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:19'!
testNumberOfResults
	"Simulating the real runners, each dummy adds two results: the serialization one and the materialization one."
	
	| benchmarks |
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks run.
	self assert: 8 = benchmarks results size! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 6/2/2011 20:03'!
testPrintCSV
	
	| benchmarks aStream |
	aStream := '' writeStream.
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks run.
	benchmarks printResultsOnCSV: aStream.
	aStream close.

	self deny: aStream contents isEmpty! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 6/2/2011 19:52'!
testPrintClassic
	
	| benchmarks aStream |
	aStream := '' writeStream.
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks run.
	benchmarks printResultsOn: aStream.
	aStream close.

	self deny: aStream contents isEmpty! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 4/12/2012 20:31'!
testUseFileStream

	| benchmarks |
	benchmarks := self fuelBenchmarksWithSmallSample.
	self shouldnt: [ 
			benchmarks
				useMultiByteFileStream;
				run ]
		raise: Error.
	benchmarks results do: [:aResult | 
		self assert: aResult successful].! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 21:02'!
testUseMemoryStream

	| benchmarks |
	benchmarks := self fuelBenchmarksWithSmallSample.
	self shouldnt: [ 
			benchmarks
				useMemoryStream;
				run ]
		raise: Error.
	benchmarks results do: [:aResult | 
		self assert: aResult successful].! !

!Time class methodsFor: '*FuelBenchmarks' stamp: 'MarianoMartinezPeck 6/30/2011 10:32'!
millisecondsToRunWithoutContextSwitch: timedBlock 
	"Answer the number of milliseconds timedBlock takes to return its value."

	| initialMilliseconds |
	initialMilliseconds := self millisecondClockValue.
	timedBlock valueNoContextSwitch.
	^self millisecondsSince: initialMilliseconds! !

Object subclass: #FLBenchmarkResult
	instanceVariableNames: 'title runner sample milliseconds successful typeOfOperation streamSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarkResult commentStamp: 'MartinDias 10/24/2011 12:15' prior: 0!
I am a result produced by FLBenchmark when running certain sample with certain runner.!

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 20:27'!
isMaterialization
	^ typeOfOperation = #materialization! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 11:04'!
isSerialization
	^ typeOfOperation = #serialization! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
milliseconds

	^ milliseconds! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
milliseconds: anObject

	milliseconds := anObject! !

!FLBenchmarkResult methodsFor: 'printing' stamp: 'MartinDias 4/6/2012 23:03'!
printOn: aStream

	aStream
		nextPutAll: title;
		nextPutAll: ' sample: ';
		nextPutAll: sample title;
		nextPutAll: ' size: ';
		nextPutAll: sample content size printString.
	
	successful 
		ifTrue: [
			aStream 
				nextPutAll: ' ms: ';
				nextPutAll: milliseconds printString.
			self isMaterialization ifTrue: [
				aStream 
					nextPut: Character cr;
					nextPutAll: 'Serialization size: ';
					nextPutAll: streamSize printString ]] 
		ifFalse: [
			aStream nextPutAll: ' FAILED' ]
! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 4/6/2012 23:02'!
runner
	^ runner! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 4/6/2012 23:02'!
runner: anObject
	runner := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 4/6/2012 23:02'!
sample
	^ sample! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 4/6/2012 23:02'!
sample: anObject
	sample := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 4/6/2012 23:03'!
sampleSize

	^ sample content size! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 4/6/2012 23:03'!
streamSize
	^ streamSize! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 4/6/2012 23:03'!
streamSize: anObject
	streamSize := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MD 12/3/2010 17:06'!
successful

	^ successful! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MD 12/3/2010 17:07'!
successful: aBoolean

	successful := aBoolean! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
title

	^ title! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
title: anObject

	title := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 11:03'!
typeOfOperation

	^ typeOfOperation ! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 11:02'!
typeOfOperation: anObject

	typeOfOperation := anObject! !

Object subclass: #FLBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarkRunner commentStamp: 'MarianoMartinezPeck 6/17/2011 14:10' prior: 0!
All FLBenchmarkRunner can be tested like this:

| aStream |
aStream :=  (MultiByteBinaryOrTextStream on: '').
FLXXXBenchmarkRunner new serialize: 'mariano' on: aStream.
aStream reset.
FLXXXBenchmarkRunner new materializeFrom: aStream.!

!FLBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:40'!
allRunners

	^ self allSubclasses 
		select: [ :aRunnerClass | aRunnerClass shouldRun ] 
		thenCollect: [ :aRunnerClass | aRunnerClass new ]! !

!FLBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:39'!
shouldRun

	^ true! !

!FLBenchmarkRunner methodsFor: 'testing' stamp: 'MartinDias 4/6/2012 22:44'!
isMain 
	^ false! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 5/29/2011 19:12'!
materializeFrom: aStream
	"Hook method for do materializing. The time for that will be included in the benchmarks results."

	self subclassResponsibility! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 16:07'!
serialize: anObject on: aStream
	"Hook method for do serializing. The time for that will be included in the benchmarks results."

	self subclassResponsibility! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/7/2012 00:10'!
title
	"Hook method for the name"

	self subclassResponsibility! !

FLBenchmarkRunner subclass: #FLBenchmarksDummyRunner
	instanceVariableNames: 'title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarksDummyRunner commentStamp: 'MartinDias 10/24/2011 12:13' prior: 0!
I am a runner for tests.!

!FLBenchmarksDummyRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:41'!
shouldRun 
	"Returns false so it's not added to the benchmarks default run'"

	^ false! !

!FLBenchmarksDummyRunner classSide methodsFor: 'instance creation' stamp: 'MartinDias 6/2/2011 19:34'!
titled: aString

	^ super new
		title: aString;
		yourself! !

!FLBenchmarksDummyRunner methodsFor: 'hooks' stamp: 'MartinDias 4/7/2012 00:10'!
title

	^ title! !

!FLBenchmarksDummyRunner methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 19:34'!
title: anObject
	title := anObject! !

FLBenchmarkRunner subclass: #FLFuelBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Runners'!

!FLFuelBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using Fuel.

See FLBenchmarks for instructions for run.!

!FLFuelBenchmarkRunner methodsFor: 'testing' stamp: 'MartinDias 4/6/2012 22:44'!
isMain 
	^ true! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:36'!
materializeFrom: aStream

	^(FLMaterializer newDefault materializeFrom: aStream) root! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:36'!
serialize: anObject on: aStream

	FLSerializer newDefault serialize: anObject on: aStream! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/7/2012 00:10'!
title

	^ 'Fuel'! !

FLBenchmarkRunner subclass: #FLFullFuelBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Runners'!

!FLFullFuelBenchmarkRunner commentStamp: 'MartinDias 9/25/2011 22:00' prior: 0!
I serialize and materialize objects for benchmarks using Fuel considering globals as much as possible.

See FLBenchmarks for instructions for run.!

!FLFullFuelBenchmarkRunner methodsFor: 'testing' stamp: 'MartinDias 4/6/2012 22:44'!
isMain 
	^ true! !

!FLFullFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
materializeFrom: aStream

	^(FLMaterializer newDefault materializeFrom: aStream) root! !

!FLFullFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
serialize: anObject on: aStream

	| aSerializer |
	aSerializer := FLSerializer newDefault.
	aSerializer analyzer useFullMappers.
	aSerializer serialize: anObject on: aStream! !

!FLFullFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/7/2012 00:10'!
title

	^ 'Full Fuel'! !

FLBenchmarkRunner subclass: #FLImageSegmentBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Runners'!

!FLImageSegmentBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using ImageSegment.

See FLBenchmarks for instructions for run.!

!FLImageSegmentBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 20:44'!
shouldRun

	self flag: #todo. "It stuck on Pharo1.3-13327 and doesn't exist on Pharo1.4."
	^false
"	^ Smalltalk includesKey: #ImageSegment"! !

!FLImageSegmentBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
materializeFrom: aStream

	^ aStream fileInObjectAndCode arrayOfRoots first! !

!FLImageSegmentBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
serialize: anObject on: aStream
	| segment |
	segment := (Smalltalk classNamed: #ImageSegment) new copyFromRootsForExport: (Array with: anObject).
	"Transcript show: 'Total outPointers: ', segment outPointers size asString; cr."
	"Transcript show: 'Total content: ', anObject size asString; cr."
	segment writeForExportOn: aStream.

	

	! !

!FLImageSegmentBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/7/2012 00:10'!
title
	^ 'ImageSegment'! !

FLBenchmarkRunner subclass: #FLReferenceStreamBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Runners'!

!FLReferenceStreamBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using ReferenceStream.

See FLBenchmarks for instructions for run.!

!FLReferenceStreamBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 02:08'!
shouldRun

	^ Smalltalk includesKey: #ReferenceStream! !

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
materializeFrom: aStream

	^ (ReferenceStream on: aStream) next! !

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
serialize: anObject on: aStream

	(ReferenceStream on: aStream) nextPut: anObject ! !

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/7/2012 00:10'!
title

	^ 'ReferenceStream'! !

FLBenchmarkRunner subclass: #FLSmartRefStreamBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Runners'!

!FLSmartRefStreamBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using SmartRefStream.

See FLBenchmarks for instructions for run.!

!FLSmartRefStreamBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 02:08'!
shouldRun

	^ Smalltalk includesKey: #SmartRefStream! !

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
materializeFrom: aStream

	^ aStream fileInObjectAndCode! !

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/12/2012 20:37'!
serialize: anObject on: aStream

	(SmartRefStream on: aStream) nextPut: anObject! !

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 4/7/2012 00:10'!
title

	^ 'SmartRefStream'! !

Object subclass: #FLBenchmarkSample
	instanceVariableNames: 'content title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarkSample commentStamp: 'MartinDias 6/1/2011 19:38' prior: 0!
I represent a benchmark sample.

The common way of create instances of me is:

FLBenchmarkSample fromSelector: #completeBinaryTree.

Also you can create a bunch of instances with:

FLBenchmarkSample basicSamples.!

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2010 10:09'!
title: aTitle content: aContent

	^ self new
		title: aTitle;
		content: aContent;
		yourself! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
content
	^ content! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/9/2011 20:49'!
content: anObject
	content := anObject! !

!FLBenchmarkSample methodsFor: 'printing' stamp: 'MartinDias 8/9/2011 15:06'!
printOn: aStream
	"Append a sequence of characters to aStream that identify the receiver."

	super printOn: aStream.
	aStream
		nextPutAll: ' title: ';
		print: title! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
title
	^ title! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
title: anObject
	title := anObject! !

Object subclass: #FLBenchmarkTotalResultForRunner
	instanceVariableNames: 'runner totalTimeSerialization totalTimeMaterialization'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarkTotalResultForRunner classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 5/17/2011 12:06'!
runner: aRunner totalTimeSerialization: millisecondsSerialization totalTimeMaterialization: millisecondsMatarialization
	^ self new initializeWithRunner: aRunner totalTimeSerialization: millisecondsSerialization totalTimeMaterialization: millisecondsMatarialization! !

!FLBenchmarkTotalResultForRunner methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/17/2011 12:07'!
initializeWithRunner: aRunner totalTimeSerialization: millisecondsSerialization totalTimeMaterialization: millisecondsMaterialization
	runner := aRunner. 
	totalTimeSerialization := millisecondsSerialization.
	totalTimeMaterialization := millisecondsMaterialization.! !

!FLBenchmarkTotalResultForRunner methodsFor: 'printing' stamp: 'MartinDias 4/7/2012 00:10'!
printResultsOn: aStream
	aStream cr.
	aStream nextPutAll:  runner title, ' total milliseconds for serialization: ', totalTimeSerialization asString; cr.
	aStream nextPutAll:  runner title, ' total milliseconds for materialization: ', totalTimeMaterialization asString; cr.
	aStream nextPutAll:  runner title, ' total milliseconds: ', (totalTimeMaterialization +  totalTimeSerialization) asString; cr.
! !

!FLBenchmarkTotalResultForRunner methodsFor: 'printing' stamp: 'MartinDias 4/7/2012 00:10'!
printResultsOn: aStream comparingToFuelResults: fuelResults
	| dif |

	self printResultsOn:  aStream.
	(fuelResults isNil or: [runner isMain]) ifTrue: [^ self ].
	aStream cr.
		(totalTimeSerialization > fuelResults totalTimeSerialization) 
			ifTrue: [
				dif := (totalTimeSerialization / fuelResults totalTimeSerialization) asFloat .
				aStream nextPutAll: 'For serialization, ', fuelResults runner title, ' is ', (dif printShowingDecimalPlaces: 2), ' times faster than ', runner title; cr.
			] ifFalse: [
				dif := ( fuelResults totalTimeSerialization / totalTimeSerialization) asFloat .
				aStream nextPutAll: 'For serialization, ', fuelResults runner title, ' is ', (dif printShowingDecimalPlaces: 2), ' times slower than ', runner title; cr.
			].
			
			(totalTimeMaterialization  > fuelResults totalTimeMaterialization ) 
			ifTrue: [
				dif := (totalTimeMaterialization / fuelResults totalTimeMaterialization) asFloat .
				aStream nextPutAll: 'For materialization, ', fuelResults runner title, ' is ', (dif printShowingDecimalPlaces: 2), ' times faster than ', runner title; cr.
			] ifFalse: [
				dif := ( fuelResults totalTimeMaterialization / totalTimeMaterialization) asFloat .
				aStream nextPutAll: 'For materialization, ', fuelResults runner title, ' is ', (dif printShowingDecimalPlaces: 2), ' times slower than ', runner title; cr.
			]
			

! !

!FLBenchmarkTotalResultForRunner methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 12:25'!
runner
	^ runner! !

!FLBenchmarkTotalResultForRunner methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 12:25'!
totalTimeMaterialization
	^ totalTimeMaterialization! !

!FLBenchmarkTotalResultForRunner methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 12:25'!
totalTimeSerialization
	^ totalTimeSerialization! !

Object subclass: #FLBenchmarks
	instanceVariableNames: 'results samples runners streamFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarks commentStamp: '<historical>' prior: 0!
I am useful for running benchmarks of Fuel. 

You can see in FLBenchmarks class>>newBasic for an example of benchmark creation. Also there are examples in 'scripts' category, in class-side. Either FLBenchmarks>>useFileStream or FLBenchmarks>>useMemoryStream indicates in which stream serialize. After running, you should use the 'printing' protocol to show results, usually on Transcript or in a CSV file.

Following, some useful examples:


- Standard set of benchmarks:

FLBenchmarks newBasic run printResultsOnTranscript.


- Just with Fuel (useful for compare between Fuel versions):

FLBenchmarks newBasicFuelOnly run printResultsOnTranscript.


- Customized sample set, just with Fuel (useful for compare between Fuel versions):

(FLBenchmarks newFuelOnlySamplesFrom: #(#strings #symbols)) run printResultsOnTranscript.


- This exports sizes of the serialized streams:

FLBenchmarks newBasic run printSizeResultsOnCSVNamed: 'sizes.csv'.


- To see how much ImageSegment gets slow when shared objects increase:

FLBenchmarks generateSharedSamplesCSV.


- To see how time evolves when a sample increases its size:

FLBenchmarks generateSubsetSamplesCSV.
!

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 4/9/2012 14:26'!
generateAllCSVFilesForPaper
	self newPrimitivesInMemory run printResultsOnCSVNamed: 'BenchForPrimitiveObjectsSerializedInMemory.csv'. 
	self newNonPrimitivesInMemory run printResultsOnCSVNamed: 'BenchForNonPrimitiveObjectsSerializedInMemory.csv'.
	self newPrimitivesInFile run printResultsOnCSVNamed: 'BenchForPrimitiveObjectsSerializedInFile.csv'.
	self newNonPrimitivesInFile run printResultsOnCSVNamed: 'BenchForNonPrimitiveObjectsSerializedInFile.csv'.
	self newPrimitivesInFile run printSizeResultsOnCSVNamed: 'BenchStreamSize.csv'.

! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 4/12/2012 20:32'!
generateFuelVersionsComparison

	| fuelVersions repetitionNumber runs |
	fuelVersions := #('Fuel-MartinDias.305' 'Fuel-MartinDias.306').
	repetitionNumber := 2.
	FileStream
		forceNewFileNamed: 'benchsClusterizationReificationBis.txt'
		do: [ :benchsFile | 
			"generate runs"
			runs := OrderedCollection new.
			repetitionNumber timesRepeat: [ runs addAll: fuelVersions ].

			"run"
			runs
				do: [ :fuelVersion | 
					(Gofer new)
						version: fuelVersion;
						load.
					FuelUtilities cleanUpEverything.
					benchsFile
						nextPutAll: ' === ' , fuelVersion , ' === ';
						cr.
					3 timesRepeat: [ Smalltalk garbageCollect ].
					FLBenchmarks new
						samples: FLSampleFactory primitiveSamples;
						runners: {FLFuelBenchmarkRunner new};
						useMultiByteFileStream;
						run;
						printResultsOn: benchsFile ]
				displayingProgress: [ :fuelVersion | 'Benchmarks batch running version ' , fuelVersion ] ]! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 4/12/2012 01:44'!
generateMiniSerializationVersionsComparison

	| versions repetitionNumber runs anObject time stream |
	versions := #('Fuel-MartinDias.622' 'Fuel-MarianoMartinezPeck.618').
	repetitionNumber := 2.
	anObject := (FLSampleFactory sampleNamed: #zipCodes) content.
	
	FileStream
		forceNewFileNamed: 'benchsMicroLatestChanges.txt'
		do: [ :benchsFile | 
			"generate runs"
			runs := OrderedCollection new.
			repetitionNumber timesRepeat: [ runs addAll: versions ].

			"run"
			runs
				do: [ :fuelVersion | 
					(Gofer new)
						version: fuelVersion;
						load.
					FuelUtilities cleanUpEverything.

					benchsFile nextPutAll: ' === ' , fuelVersion , ' === '; cr.
					
					3 timesRepeat: [ Smalltalk garbageCollect ].

					time := [ 
						stream := (ByteArray new: 100) writeStream.
						FLSerializer newDefault 
						serialize: anObject 
						on: stream ] bench.
					
					benchsFile 
						nextPutAll: 'size: ';
						nextPutAll: stream contents size printString; 
						cr;
						nextPutAll: 'bench: ';
						nextPutAll: time printString; 
						cr. 
						
				] displayingProgress: [ :fuelVersion | 'Benchmarks batch running version ' , fuelVersion ] ]! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 4/12/2012 01:44'!
generateSharedSamplesCSV

   	| samples |
	samples := OrderedCollection new.

	0 to: 100 by: 20 do: [ :percent |
		| sample subsetSize aSubsetOfSample |
		sample := FLSampleFactory sampleNamed: #points.
		sample title: (percent asString) , $% asString.  
		samples add: sample.
       
		subsetSize := (sample content size * percent) // 100.
		aSubsetOfSample := sample content asOrderedCollection first: subsetSize.
		Smalltalk globals at: ('FuelBenchmarksShared', percent printString) asSymbol put: aSubsetOfSample ].
   
	FLBenchmarks new
		samples: samples;
		runners: { FLFuelBenchmarkRunner new. FLSmartRefStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. };
		useMemoryStream;
		run;
		printResultsOnCSVNamed: 'imageSegmentSharedObjectsBenchmarks.csv'. . ! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 4/12/2012 01:44'!
generateSubsetSamplesCSV

   	| samples |
	samples := OrderedCollection new.

	0 to: 100 by: 10 do: [ :percent |
		| sample subsetSize aSubsetOfSample |
		sample := FLSampleFactory sampleNamed: #points.
		subsetSize := (sample content size * percent) // 100.
		aSubsetOfSample := sample content asOrderedCollection first: subsetSize. 
		
		sample title: (percent asString) , $% asString.
		sample content: aSubsetOfSample.
		samples add: sample].
   
	FLBenchmarks new
		samples: samples;
		runners: (Array with: FLFuelBenchmarkRunner new with: FLSmartRefStreamBenchmarkRunner new with: FLImageSegmentBenchmarkRunner new);
		useMemoryStream;
		run;
		printResultsOnCSVNamed: 'subsetBenchmarks.csv'. ! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/12/2012 20:31'!
newBasic

	^ self new 
		samples: FLSampleFactory primitiveSamples;
		runners: FLBenchmarkRunner allRunners;
		useMultiByteFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/12/2012 20:31'!
newBasicFuelOnly

	^ self new 
		samples: FLSampleFactory primitiveSamples;
		runners: (Array with: FLFuelBenchmarkRunner new with: FLFullFuelBenchmarkRunner new);
		useMultiByteFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/12/2012 20:31'!
newNonPrimitivesInFile

	^ self new 
		samples: FLSampleFactory nonPrimitiveSamples;
		runners: FLBenchmarkRunner allRunners;
		useMultiByteFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/12/2012 01:39'!
newNonPrimitivesInMemory

	^ self new 
		samples: FLSampleFactory nonPrimitiveSamples;
		runners: FLBenchmarkRunner allRunners;
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/12/2012 20:31'!
newPrimitivesInFile

	^ self new 
		samples: FLSampleFactory primitiveSamples;
		runners: FLBenchmarkRunner allRunners;
		useMultiByteFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/12/2012 01:38'!
newPrimitivesInMemory

	^ self new 
		samples: FLSampleFactory primitiveSamples;
		runners: FLBenchmarkRunner allRunners;	
		useMemoryStream;
		yourself.! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 4/7/2012 00:10'!
addResultRunning: aBlockClosure runner: aRunner sample: aSample typeOfOperation: aSymbol streamSize: aNumberOfBytes

	| timeToRun successful |
	[ self warmUp: aBlockClosure.
	timeToRun := Time millisecondsToRunWithoutContextSwitch: aBlockClosure. 
	successful := true ] 
		on: Error
		do: [ timeToRun := 0. successful := false ].

	results add: (FLBenchmarkResult new
		title: aRunner title, ' ', aSymbol;
		runner: aRunner;
		sample: aSample;
		milliseconds: timeToRun;
		successful: successful;
		typeOfOperation: aSymbol;
		streamSize: aNumberOfBytes;
		yourself)! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 4/6/2012 22:56'!
addResultRunningMaterialization: aBlockClosure runner: aRunner sample: aSample streamSize: aNumberOfBytes

	^ self addResultRunning: aBlockClosure runner: aRunner sample: aSample typeOfOperation: #materialization streamSize: aNumberOfBytes
! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 4/6/2012 22:56'!
addResultRunningSerialization: aBlockClosure runner: aRunner sample: aSample

	^ self addResultRunning: aBlockClosure runner: aRunner sample: aSample typeOfOperation: #serialization streamSize: 0
! !

!FLBenchmarks methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 7/1/2011 14:11'!
initialize 

	super initialize.
	results := OrderedCollection new.
	runners := OrderedCollection new.
	samples := OrderedCollection new.! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:46'!
printResultsOn: aStream

	(FLBenchmarksClassicPrinter on: aStream) print: self
! !

!FLBenchmarks methodsFor: 'printing' stamp: 'mariano 6/3/2011 10:07'!
printResultsOnCSV

	FileStream 
		forceNewFileNamed: 'benchmarks.csv'
		do: [ :aFile | self printResultsOnCSV: aFile ]
! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 19:44'!
printResultsOnCSV: aStream

	(FLBenchmarksCSVPrinter on: aStream) print: self! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/17/2011 19:04'!
printResultsOnCSVNamed: aFileName

	FileDirectory default 
		forceNewFileNamed: aFileName 
		do: [ :aFile | (FLBenchmarksCSVPrinter on: aFile) print: self ]! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:53'!
printResultsOnTranscript

	self printResultsOn: Transcript.
	Transcript flush.! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/17/2011 22:19'!
printSizeResultsOnCSVNamed: aFileName

	FileDirectory default 
		forceNewFileNamed: aFileName 
		do: [ :aFile | (FLBenchmarksSizeCSVPrinter on: aFile) print: self ]! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/5/2011 16:07'!
readStreamNamed: aName

	^ readStreamFactoryBlock value: aName! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 00:39'!
results

	^ results! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
results: anObject
	results := anObject! !

!FLBenchmarks methodsFor: 'running' stamp: 'MartinDias 4/12/2012 02:03'!
run

	self runners 
		do: [ :aRunner | self run: aRunner ] 
		displayingProgress: [ :aRunner | 'Benchmarks runner: ', aRunner title ]! !

!FLBenchmarks methodsFor: 'running' stamp: 'MartinDias 4/12/2012 02:03'!
run: aRunner

	self samples 
		do: [ :aSample | self run: aRunner with: aSample ] 
		displayingProgress: [ :aSample | 'Sample: ', aSample title ]
! !

!FLBenchmarks methodsFor: 'running' stamp: 'MartinDias 4/12/2012 20:26'!
run: aRunner with: aSample

	| serializationSize |
	self 
		addResultRunningSerialization: [
			self streamFactory writeStreamDo: [ :aStream |
				aRunner serialize: aSample content on: aStream.
				serializationSize := aStream reset size. ] ]
		runner: aRunner 
		sample: aSample.
	
	self 
		addResultRunningMaterialization: [
			self streamFactory readStreamDo: [ :aStream |			
				aRunner materializeFrom: aStream ] ]
		runner: aRunner 
		sample: aSample 
		streamSize: serializationSize.! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:19'!
runners

	^ runners! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
runners: anObject
	runners := anObject! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
samples
	^ samples! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
samples: anObject
	samples := anObject! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 20:21'!
streamFactory
	^ streamFactory! !

!FLBenchmarks methodsFor: 'stream-strategies' stamp: 'MartinDias 4/12/2012 20:19'!
useMemoryStream

	streamFactory := FLMultiByteStreamStrategy new! !

!FLBenchmarks methodsFor: 'stream-strategies' stamp: 'MartinDias 4/12/2012 20:20'!
useMultiByteFileStream

	streamFactory := FLFileStreamStrategy newWithMultiByteFileStream! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 4/6/2012 23:50'!
warmUp: aBlockClosure
	"In CogVM the first time a method is used, it is executed in the old slow way, that is, interpreted, and it is added to the cache.. The second time a method is used, that means, when it is found in the cache, Cog converts that method to machine code. However, to do that, extra time is needed. Only the third time the method will be executed as machine code and without extra effort. So, for the samples we run 2 times the same sample so that to be sure to be always in the same condition."
	"A GC can occur at any time and we cannot control it. However, doing a GC before the probe, we minimize the chances of having a GC during the probe and put noise in the measure because of the GC time."
	
	2 timesRepeat: [ aBlockClosure value ]. 
	3 timesRepeat: [ Smalltalk garbageCollect ].! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/5/2011 16:07'!
writeStreamNamed: aName

	^ writeStreamFactoryBlock value: aName! !

Object subclass: #FLBenchmarksCSVPrinter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarksCSVPrinter commentStamp: 'MartinDias 6/2/2011 20:48' prior: 0!
My goal is export benchmark results to a CSV file. Useful for creating a chart with tools like OpenOffice or Excel.

An example of use:

FLBenchmarks 
	newBasic
	run; 
	printResultsOnCSV.!

!FLBenchmarksCSVPrinter classSide methodsFor: 'instance creation' stamp: 'MartinDias 11/11/2010 18:01'!
on: aStream

	^ self basicNew 
		initialize;
		stream: aStream;
		yourself! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 4/6/2012 23:08'!
columnNamesFrom: benchmarkResultsGroupedByRow
	
	^ (self columnOrderedResults: benchmarkResultsGroupedByRow anyOne) 
		collect: [ :aResult | aResult sample title ]! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 4/6/2012 23:08'!
columnOrderedResults: results
	
	^ results sort: [ :x :y | (x sample title) <= (y sample title) ]! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 18:56'!
dataFrom: aResult

	^ aResult milliseconds printString! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/2/2011 17:19'!
nextPutColumnNameRowOf: aBenchmarkResults
	"Writes the column names row"

	stream nextPutAll: 'run name'.
	(self columnNamesFrom: aBenchmarkResults)
		do: [ :aColumnName | stream nextPutAll: ', ', aColumnName ].
	stream cr.! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'mariano 6/3/2011 11:19'!
nextPutDataRowsOf: benchmarkResultsGroupedByRow
	"Writes the data rows"
	
	self nextPutSerializationDataRowsOf: benchmarkResultsGroupedByRow.
	self nextPutMaterializationDataRowsOf: benchmarkResultsGroupedByRow.

! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'mariano 6/3/2011 11:21'!
nextPutMaterializationDataRowsOf: benchmarkResultsGroupedByRow
	
	self nextPutOperation: 'Materialization' dataRowsOf: benchmarkResultsGroupedByRow
	

! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 18:56'!
nextPutOperation: anOperationName dataRowsOf: benchmarkResultsGroupedByRow
	| titles resultsInRow |
	
	stream cr.
	self nextPutColumnNameRowOf: benchmarkResultsGroupedByRow.
	
	titles := (benchmarkResultsGroupedByRow keys select: [:each | each includesSubString: anOperationName ]) sort: [:a :b | a < b ].
	
	titles do: [:title |

		"Grouping attribute"
		stream nextPutAll: title.
		
		resultsInRow := benchmarkResultsGroupedByRow at: title.
		"Data to show in cells"
		(self columnOrderedResults: resultsInRow) do: [ :aResult | 
			stream nextPutAll: ', ', (self dataFrom: aResult) ].
		stream cr.
	].
	stream cr ! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'mariano 6/3/2011 11:21'!
nextPutSerializationDataRowsOf: benchmarkResultsGroupedByRow
	
	self nextPutOperation: 'Serialization' dataRowsOf: benchmarkResultsGroupedByRow
	
! !

!FLBenchmarksCSVPrinter methodsFor: 'printing' stamp: 'mariano 6/3/2011 11:24'!
print: benchmarks

	| resultsGroupedByRow |

	resultsGroupedByRow := benchmarks results groupedBy: [ :x | x title ].
		
	self nextPutDataRowsOf: resultsGroupedByRow.! !

!FLBenchmarksCSVPrinter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:36'!
stream

	^ stream! !

!FLBenchmarksCSVPrinter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:36'!
stream: anObject

	stream := anObject! !

FLBenchmarksCSVPrinter subclass: #FLBenchmarksSizeCSVPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarksSizeCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 4/6/2012 23:03'!
dataFrom: aResult

	^ aResult streamSize printString! !

!FLBenchmarksSizeCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 18:53'!
nextPutDataRowsOf: benchmarkResultsGroupedByRow
	"Writes the data rows"
	
	self nextPutMaterializationDataRowsOf: benchmarkResultsGroupedByRow.

! !

Object subclass: #FLBenchmarksClassicPrinter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Core'!

!FLBenchmarksClassicPrinter commentStamp: 'MartinDias 6/2/2011 20:54' prior: 0!
My goal is export benchmark results in a human readable format.

An example of use:

FLBenchmarks 
	newBasic
	run; 
	printResultsOnTranscript

!

!FLBenchmarksClassicPrinter classSide methodsFor: 'instance creation' stamp: 'MartinDias 6/2/2011 19:54'!
on: aStream

	^ self basicNew 
		initialize;
		stream: aStream;
		yourself! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:53'!
print: benchmarks 

	self printIndividualResults: benchmarks results.
	self printTotals: benchmarks results.! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 23:01'!
printIndividualResults: results

	results do: [ :aResult | 
			aResult printOn: stream.
			stream cr. ]! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 4/6/2012 23:08'!
printTotals: results

	| resultsGroupedByRunner totalResults fuelTotalResultOrNil |
	resultsGroupedByRunner := results groupedBy: [ :x | x runner ].
	
	totalResults := resultsGroupedByRunner 
		collect: [ :runnerResults | self totalResultFor: runnerResults anyOne runner in: runnerResults ].

	fuelTotalResultOrNil := 
		totalResults
			detect: [:aTotalResult | aTotalResult runner isMain] 
			ifNone: [ nil ].

	totalResults do: [ :totalResult |
		totalResult 
			printResultsOn: stream 
			comparingToFuelResults: fuelTotalResultOrNil ]! !

!FLBenchmarksClassicPrinter methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 19:54'!
stream
	^ stream! !

!FLBenchmarksClassicPrinter methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 19:54'!
stream: anObject
	stream := anObject! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:27'!
totalResultFor: aRunner in: results

	| totalTimeSerialization totalTimeMaterialization |
	totalTimeSerialization := 
		(results select: [ :aResult | aResult isSerialization ]) 
			sum: [ :aResult | aResult milliseconds ].
	totalTimeMaterialization := 
		(results select: [ :aResult | aResult isMaterialization ]) 
			sum: [ :aResult | aResult milliseconds ].

	^ FLBenchmarkTotalResultForRunner 
		runner: aRunner 
		totalTimeSerialization: totalTimeSerialization 
		totalTimeMaterialization: totalTimeMaterialization.
	! !

Object subclass: #FLDictionaryBenchmarks
	instanceVariableNames: 'dictionaryClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Other'!

!FLDictionaryBenchmarks commentStamp: 'MartinDias 5/30/2011 03:24' prior: 0!
Print it on each line:

FLDictionaryBenchmarks resultsOfHash. 
FLDictionaryBenchmarks resultWithFastHashSample. 
FLDictionaryBenchmarks resultWithSlowHashSample. !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/24/2011 16:51'!
newWith: aDictionaryClass

	^self basicNew 
		initializeWith: aDictionaryClass;
		yourself! !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/19/2011 15:40'!
resultWithFastHashSample
	
	|benchs|
	benchs := (self newWith: Dictionary).
	^benchs runWith: benchs smallintegersAndStringsSampleDictionary. ! !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/19/2011 15:40'!
resultWithSlowHashSample
	
	|benchs|
	benchs := (self newWith: Dictionary).
	^benchs runWith: benchs stringsAndSmallintegersSampleDictionary. ! !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/19/2011 15:54'!
resultsOfHash

	| aSmallIntegerCollection aStringCollection |
	aSmallIntegerCollection := (1 to: (1 << 19)) asOrderedCollection.
	aStringCollection := ((1 to: (1 << 19)) collect: [:x | x printString ]) asOrderedCollection.

	^ 'small integers: ', [ aSmallIntegerCollection do: [:x | x hash ] ] timeToRun printString, 'ms. ',
	'strings: ', [ aStringCollection do: [:x | x hash ] ] timeToRun printString, 'ms.'.
! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MartinDias 5/19/2011 15:48'!
basicCopyOf: anObject
	"Code extracted  from Object>>shallowCopy"

	|class newObject index |

	class := anObject class.
	class isVariable
		ifTrue: 
			[index := anObject basicSize.
			newObject := class basicNew: index.
			[index > 0]
				whileTrue: 
					[newObject basicAt: index put: (anObject basicAt: index).
					index := index - 1]]
		ifFalse: [newObject := class basicNew].
	index := class instSize.
	[index > 0]
		whileTrue: 
			[newObject instVarAt: index put: (anObject instVarAt: index).
			index := index - 1].
	^ newObject! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MartinDias 5/19/2011 15:56'!
copyRecreating: sourceDictionary

	| copy |
	copy := sourceDictionary class new: sourceDictionary size.
	
	sourceDictionary associationsDo: [:assoc | 
		copy at: assoc key put: assoc value ].
! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MartinDias 5/19/2011 15:56'!
copyRehashing: sourceDictionary 

	(self basicCopyOf: sourceDictionary) rehash! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MarianoMartinezPeck 10/24/2011 16:52'!
initializeWith: aDictionaryClass
	super initialize.
	dictionaryClass := aDictionaryClass! !

!FLDictionaryBenchmarks methodsFor: 'testing' stamp: 'MartinDias 5/19/2011 15:28'!
runWith: sourceDictionary 

	^ #(copyRecreating: copyRehashing:)
		inject: '' 
		into: [:result :selector | 
			result, 
			selector printString, 
			' run ms = ', 
			[self perform: selector with: sourceDictionary] 
				timeToRun printString ].
! !

!FLDictionaryBenchmarks methodsFor: 'testing' stamp: 'MartinDias 5/19/2011 15:51'!
smallintegersAndStringsSampleDictionary

	| sourceDictionary|
	sourceDictionary := dictionaryClass new.
	1 to: (1 << 19) do:  [:aNumber | sourceDictionary at: aNumber put: aNumber printString ] .
	^sourceDictionary ! !

!FLDictionaryBenchmarks methodsFor: 'testing' stamp: 'MartinDias 5/19/2011 15:51'!
stringsAndSmallintegersSampleDictionary

	| sourceDictionary|
	sourceDictionary := dictionaryClass new.
	1 to: (1 << 19) do:  [:aNumber | sourceDictionary at: aNumber printString put: aNumber ] .
	^sourceDictionary ! !

Object subclass: #FLExecutionBenchmarks
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Other'!

!FLExecutionBenchmarks commentStamp: 'MartinDias 9/17/2011 19:55' prior: 0!
Open Trascript and then evaluate: 

FLExecutionBenchmark benchs.!

!FLExecutionBenchmarks classSide methodsFor: 'benchmarking' stamp: 'MartinDias 9/17/2011 20:04'!
benchs

	(self class methodsInCategory: 'executing')
		do: [:m |
			Transcript 
				cr; 
				show: m printString;
				show: '->'.
			[self perform: m] bench printOn: Transcript ].
	Transcript flush.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:01'!
block1

	^[ Integer ] value.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:01'!
block2

	^[ Smalltalk globals at: #Integer ] value.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:00'!
block3

	^ ([:x | x globals ] value: Smalltalk) at: #Integer.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:00'!
block4

	^ [:y :z| y at: z ] 
		value: ([:x | x globals ] value: Smalltalk)
		value: #Integer
! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 19:47'!
compiled1

	^ Integer! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 19:46'!
compiled2

	^ Smalltalk globals at: #Integer! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 19:48'!
perform1

	^ (Smalltalk 
		perform: #globals) 
		perform: #at: with: #Integer! !

Object subclass: #FLFixedDegreeGraphNode
	instanceVariableNames: 'child1 child2 child3 child4 child5 child6 child7 child8'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLFixedDegreeGraphNode commentStamp: '<historical>' prior: 0!
I am useful to benchmark serialization performance. My children are pointed through fixed references.!

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child1
	^ child1! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child1: anObject
	child1 := anObject! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child2
	^ child2! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child2: anObject
	child2 := anObject! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child3
	^ child3! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child3: anObject
	child3 := anObject! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child4
	^ child4! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child4: anObject
	child4 := anObject! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child5
	^ child5! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child5: anObject
	child5 := anObject! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child6
	^ child6! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child6: anObject
	child6 := anObject! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child7
	^ child7! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child7: anObject
	child7 := anObject! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child8
	^ child8! !

!FLFixedDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 23:36'!
child8: anObject
	child8 := anObject! !

Object subclass: #FLSampleFactory
	instanceVariableNames: 'problemSize selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLSampleFactory commentStamp: '<historical>' prior: 0!
I am a factory for samples, used to benchmark serialization performance.!

FLSampleFactory subclass: #FLArrayedCollectionSampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLArrayedCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
bitmaps: eachSize

	^ (1 to: self problemSize // eachSize)
		collect: [ :i | 
			Bitmap withAll: (1 + i to: eachSize + i) ]! !

!FLArrayedCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
byteArrays: eachSize

	^ (1 to: self problemSize // eachSize)
		collect: [ :each | 
			(1 to: eachSize) collect: [:i | (each + i \\ 256)] as: ByteArray ]! !

!FLArrayedCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
byteStrings: eachSize

	^ (self byteArrays: eachSize)
		collect: [ :each | each asString ]! !

!FLArrayedCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
largeBitmaps

	^ self bitmaps: 1024 * 1024! !

!FLArrayedCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
largeByteStrings

	^ self byteStrings: 2048! !

!FLArrayedCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
largeWideStrings

	^ self wideStrings: 2048! !

!FLArrayedCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
smallBitmaps

	^ self bitmaps: 32 * 32! !

!FLArrayedCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
smallByteStrings

	^ self byteStrings: 10! !

!FLArrayedCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
smallWideStrings

	^ self wideStrings: 10! !

!FLArrayedCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
symbols
	
	^ self smallByteStrings collect: [:each | each asSymbol ]! !

!FLArrayedCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
wideStrings: eachSize

	^ (self byteStrings: eachSize)
		collect: [ :each | each asWideString ]! !

FLSampleFactory subclass: #FLCollectionSampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
byteCharacters

	^ (1 to: self problemSize)
		collect: [ :i | Character value: (i \\ 256) ]! !

!FLCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
denseCollections
	
	^ self denseCollectionsOfAll: self fewCollectionClasses! !

!FLCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
denseCollectionsOfAll: anArrayOfCollectionClasses
	
	^ (1 to: self problemSize) 
		collect: [ :i | (anArrayOfCollectionClasses atWrap: i) denseSample ]! !

!FLCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
emptyCollections
	
	^ self emptyCollectionsOfAll: self fewCollectionClasses! !

!FLCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
emptyCollectionsOfAll: anArrayOfCollectionClasses
	
	^ (1 to: self problemSize) 
		collect: [ :i | (anArrayOfCollectionClasses atWrap: i) new ]! !

!FLCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
fewCollectionClasses
	^ Array
		with: Dictionary
		with: IdentityDictionary
		with: Set
		with: IdentitySet
		with: OrderedCollection! !

!FLCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
sparseCollections

	^ self sparseCollectionsOfAll: self fewCollectionClasses! !

!FLCollectionSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
sparseCollectionsOfAll: anArrayOfCollectionClasses
	
	^ (1 to: self problemSize) 
		collect: [ :i | (anArrayOfCollectionClasses atWrap: i) sparseSample ]! !

!FLCollectionSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
wideCharacters

	^ self byteCharacters collectWithIndex: [:char :i | (char asciiValue + 256 + (i \\ 16r3FFFFF)) asCharacter ]! !

FLSampleFactory subclass: #FLCommonSampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLCommonSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
dateAndTimes

	| initialTime initialDate |
	initialTime := Time fromSeconds: 76020.
	initialDate := Date fromSeconds: 3492288000.
	^ (1 to: self problemSize) 
		collect: [:index |
			DateAndTime 
				date: (initialDate subtractDays: index) 
				time: (initialTime addSeconds: index) ].
! !

!FLCommonSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
dates

	| initialDate |
	initialDate := Date fromSeconds: 3492288000.
	^ (1 to: self problemSize) 
		collect: [:index | initialDate subtractDays: index ]! !

!FLCommonSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
durations
	^ self times collect: [:each | each asDuration]! !

!FLCommonSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
points

	^ (1 to: self problemSize) 
		collect: [ :i | i @ (i+1) ]! !

!FLCommonSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
rectangles

	| step |
	step := 4.
	^ (1 to: self problemSize * step by: step) 
		collect: [ :i | i@(i+1) corner: (i+2)@(i+3) ]
! !

!FLCommonSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
times

	| initialTime |
	initialTime := Time fromSeconds: 76020.
	^ (1 to: self problemSize) 
		collect: [ :index | initialTime addSeconds: index ].
! !

FLSampleFactory subclass: #FLFuelSpecificSampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLFuelSpecificSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
hookSubstitutions

	^ (1 to: self problemSize) 
		collect: [ :i | FLSubstitutedObject with: i ].
! !

FLSampleFactory subclass: #FLNumberSampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
floats

	| temp |
	temp := 1.1.
	^ (1 to: self problemSize) collect: [ :i | temp := temp negated. i * temp ]
! !

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
largeIntegers

	^ self positiveLargeIntegers 
		collectWithIndex: [:each :index | 
			index even 
				ifTrue: [ each negated ]
				ifFalse: [ each ] ]! !

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
negativeLargeIntegers

	^ self positiveLargeIntegers collect: [:each | each negated ]! !

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
positiveLargeIntegers

	| maxLarge minLarge |
	minLarge := SmallInteger maxVal + 1.
	maxLarge := (SmallInteger maxVal + 1) << 32.
	^ (minLarge to: maxLarge by: (maxLarge - minLarge) // self problemSize) asArray! !

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
scaledDecimals

	^ self floats collect: [:f | f asScaledDecimal ]! !

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
smallIntegers

	^ (SmallInteger minVal to: SmallInteger maxVal by: 2 * SmallInteger maxVal // self problemSize) asArray! !

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
smallIntegers16Bits

	| number |
	number := 1 << 15.
	^ (number negated to: number by: 2 * number / self problemSize) collect: [ :n | n floor ]! !

!FLNumberSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
smallIntegers8Bits

	| number |
	number := 1 << 7.
	^ (number negated to: number by: 2 * number / self problemSize) collect: [ :n | n floor ]! !

!FLSampleFactory classSide methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 01:28'!
allSampleSelectors

	^ self methodsInCategory: 'samples'! !

!FLSampleFactory classSide methodsFor: 'sample creation' stamp: 'MartinDias 4/12/2012 01:44'!
allSamples

	^self samplesNamed: self allSampleSelectors! !

!FLSampleFactory classSide methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 01:51'!
defaultProblemSize
	^ 1 << 8! !

!FLSampleFactory classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/9/2012 07:13'!
new

	^ self basicNew
		initializeWith: self defaultProblemSize;
		yourself.! !

!FLSampleFactory classSide methodsFor: 'sample creation' stamp: 'MartinDias 4/12/2012 01:43'!
nonPrimitiveSamples

	^ {FLCollectionSampleFactory. FLCommonSampleFactory. FLStructureSampleFactory. FLSystemSampleFactory}
		gather: [:aFactory | aFactory allSamples ]! !

!FLSampleFactory classSide methodsFor: 'sample creation' stamp: 'MartinDias 4/12/2012 01:37'!
primitiveSamples

	^ {FLNumberSampleFactory. FLArrayedCollectionSampleFactory}
		gather: [:aFactory | aFactory allSamples ]! !

!FLSampleFactory classSide methodsFor: 'sample creation' stamp: 'MartinDias 4/12/2012 01:48'!
sampleNamed: aSelector

	^ self new 
		selector: aSelector;
		asSample! !

!FLSampleFactory classSide methodsFor: 'sample creation' stamp: 'MartinDias 4/12/2012 01:44'!
samplesNamed: aCollectionOfSampleSelectors
	"Answer a collection of instances of me, with the selectors. Each selector should define a class-side method."

	^aCollectionOfSampleSelectors
		collect: [ :aSelector | self sampleNamed: aSelector ]! !

!FLSampleFactory methodsFor: 'building' stamp: 'MartinDias 4/12/2012 02:20'!
asSample

	^ FLBenchmarkSample 
		title: selector asString
		content: (self perform: selector) ! !

!FLSampleFactory methodsFor: 'initialize-release' stamp: 'MartinDias 4/9/2012 07:10'!
initializeWith: aNumber

	self initialize.
	problemSize := aNumber.! !

!FLSampleFactory methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 07:09'!
problemSize
	^ problemSize! !

!FLSampleFactory methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 00:37'!
problemSize: anObject
	problemSize := anObject! !

!FLSampleFactory methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 00:47'!
selector
	^ selector! !

!FLSampleFactory methodsFor: 'accessing' stamp: 'MartinDias 4/12/2012 00:47'!
selector: anObject
	selector := anObject! !

FLSampleFactory subclass: #FLStructureSampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLStructureSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:53'!
completeBinaryTree 

	^ Array with: (self completeBinaryTree: self problemSize)! !

!FLStructureSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
completeBinaryTree: index

	| children left right |
	children := OrderedCollection new.
	left := index // 2 - 1.
	right := index // 2.
	left > 0 ifTrue: [ children add: (self completeBinaryTree: left) ].
	right > 0 ifTrue: [ children add: (self completeBinaryTree: right) ].

	^ FLVariableDegreeGraphNode newWith: children asArray! !

!FLStructureSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
completeGraph
	"Answe ra complete graph of aproximately problemSize edges. A complete graph of n vertices has n*(n-1)/2 edges."

	| n vertices |
	n := (self problemSize * 2) sqrt rounded.
	vertices := (1 to: n) collect: [ :index | FLVariableDegreeGraphNode new ] as: Array.
	
	vertices do: [ :aNode | 
		aNode children: (vertices copyWithout: aNode) ].
	
	^ vertices! !

!FLStructureSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
regularGraphOfFixedDegree
	"Answer a regular graph of aproximately problemSize vertices."

	| n vertices |
	n := (self problemSize // 8) * 8.
	vertices := (1 to: n) collect: [ :index | FLFixedDegreeGraphNode new ] as: Array.

	1 to: n do: [ :index | 
		| aNode |
		aNode := vertices at: index.
		aNode child1: (vertices atWrap: index + 1).
		aNode child2: (vertices atWrap: index + 2).
		aNode child3: (vertices atWrap: index + 3).
		aNode child4: (vertices atWrap: index + 4).
		aNode child5: (vertices atWrap: index + 5).
		aNode child6: (vertices atWrap: index + 6).
		aNode child7: (vertices atWrap: index + 7).
		aNode child8: (vertices atWrap: index + 8).
	 ].
	
	^ vertices! !

FLSampleFactory subclass: #FLSystemSampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLSystemSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
compiledMethods

	| methods |
	methods := self fewSystemClasses inject: Array new into: [ :array :aClass | array, aClass methods ].
	
	^ (1 to: self problemSize)
		collect: [ :i | (methods atWrap: i) copy ]! !

!FLSystemSampleFactory methodsFor: 'private' stamp: 'MartinDias 4/12/2012 01:18'!
fewSystemClasses

	^ #('System' 'Collections' 'SUnit') 
		inject: Array new 
		into: [:anArray :aName | anArray, (SystemNavigation default allClassesInPackageNamed: aName) ]! !

!FLSystemSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
globalClasses

	| classes |	
	classes := self fewSystemClasses.

	^ (1 to: self problemSize) collect: [ :i | classes at: 1 + (i \\ 150) ].! !

!FLSystemSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
globals

	| globals |	
	globals := FLAnalyzer defaultGlobalSymbols collect: [ :s | Smalltalk globals at: s ].

	^ (1 to: self problemSize) collect: [ :i | globals atWrap: i ].! !

!FLSystemSampleFactory methodsFor: 'samples' stamp: 'MartinDias 4/12/2012 01:18'!
methodDictionaries

	| methodDicts |
	methodDicts := self fewSystemClasses collect: [ :aClass | aClass methodDict ].
	
	^ (1 to: self problemSize)
		collect: [ :i | (methodDicts atWrap: i) copy ]! !

!FLSystemSampleFactory methodsFor: 'failing' stamp: 'MartinDias 4/12/2012 01:18'!
nilAndBooleans
	
	^ (1 to: self problemSize) 
		collect: [ :index | #(nil true false) atWrap: index ]! !

Object subclass: #FLSubstitutedObject
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLSubstitutedObject commentStamp: '<historical>' prior: 0!
I am useful to benchmark serialization performance. I am substituted by my contents during serialization.!

!FLSubstitutedObject classSide methodsFor: 'instance creation' stamp: 'MartinDias 1/7/2012 14:28'!
with: anObject

	^ self basicNew
		initializeWith: anObject;
		yourself.! !

!FLSubstitutedObject methodsFor: 'hooks' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper
	^ aGeneralMapper visitSubstitution: self by: contents! !

!FLSubstitutedObject methodsFor: 'initialize-release' stamp: 'MartinDias 1/7/2012 14:27'!
initializeWith: anObject

	self initialize.
	contents := anObject.! !

Object subclass: #FLVariableDegreeGraphNode
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLVariableDegreeGraphNode commentStamp: '<historical>' prior: 0!
I am useful to benchmark serialization performance. My children are pointed through variable references.!

!FLVariableDegreeGraphNode classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/10/2012 00:02'!
new
 
	^ self newWith: #()! !

!FLVariableDegreeGraphNode classSide methodsFor: 'instance creation' stamp: 'MartinDias 4/10/2012 00:03'!
newWith: aCollectionOfNodes

	^self basicNew
		initialize;
		children: aCollectionOfNodes;
		yourself.! !

!FLVariableDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 03:56'!
children
	^ children! !

!FLVariableDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 03:56'!
children: anObject
	children := anObject! !

!FLVariableDegreeGraphNode methodsFor: 'accessing' stamp: 'MartinDias 4/9/2012 04:13'!
treeSize

	^ 1 + (self children inject: 0 into: [ :sum :node | sum + node treeSize ])! !
